Package: CloneSeeker
Version: 1.0.16
Date: 2026-02-10
Title: Seeking and Finding Clones in Copy Number and Sequencing Data
Authors@R: c(person(given = "Mark", family = "Zucker", role = "aut"),
	    person(given = c("Kevin", "R."), family = "Coombes",
                   role = c("aut", "cre"), email = "krc@silicovore.com"))
Description: Defines the classes and functions used to simulate and
  to analyze data sets describing copy number variants and,
  optionally, sequencing mutations in order to detect clonal subsets.
  See Zucker et al. (2019) <doi:10.1093/bioinformatics/btz057>.
Depends: R (>= 3.0)
Imports: methods, graphics, combinat, mc2d, quantmod
License: Apache License (== 2.0)
URL: http://oompa.r-forge.r-project.org/
Author: Mark Zucker [aut],
  Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: R-Forge
Repository/R-Forge/Project: clonefinder
Repository/R-Forge/Revision: 196
Repository/R-Forge/DateTimeStamp: 2026-07-06 13:08:50
Date/Publication: 2026-07-06 13:08:50
NeedsCompilation: no
Packaged: 2026-07-06 13:24:22 UTC; rforge
